
public class TGauss {
	double[][] m;
	double[] x;
	double[] y;
	int[] index;
	int maxOrder;
	
	public TGauss(int size, double[][] matrix) {
		m = matrix;
		maxOrder = size;
		x = new double[maxOrder];
		y = new double[maxOrder];
		index = new int[maxOrder];
	}
	
	
	public void Pivot(int n)
    {
		double tempD = 0;
        int j;
        if (n < maxOrder)
        {
            for (j = 0; j < maxOrder; j++)
            {
                tempD = tempD + Math.abs(m[n][j]);
            }
            tempD = tempD / maxOrder;
            if (tempD != 0)
            {
                for (j = 0; j < maxOrder; j++)
                {
                    m[n][j] = m[n][j] / tempD;
                }
                y[n] = y[n] / tempD;
            }
        }
    }
	

    void SwitchColumns(int n)
    {
        double tempD;
        int i, j, temoI;
        for (i = n; i <= maxOrder - 2; i++)
        {
            for (j = 0; j <= maxOrder - 1; j++)
            {
                tempD = m[j][i];
                m[j][i] = m[j][i + 1];
                m[j][i + 1] = tempD;
            }
            temoI = index[i];
            index[i] = index[i+1];
            index[i + 1] = temoI;
        
        }
    }

    void Reorder()
    {
        int i;
        double[] tempVal = new double[maxOrder];
        for (i = 0; i < maxOrder; i++)
        {
            if (index[i] < maxOrder)
            {
                tempVal[index[i]] = x[i];
            }
        }
        for (i = 0; i < maxOrder; i++)
        {
           x[i] = tempVal[i];
        }
    }
	
	public boolean Eliminate(int empties)
    {
        int i, k, l;
        boolean calcError = false;
        i = 1;
        for (l = 0; l < maxOrder; l++)
        {
            index[l] = l;
            x[l] = 0.0;
            y[l] = 0.0;
        }
        for (k = 0; k <= maxOrder - 1 - empties; k++)
        {
            l = k + 1;
            while ((Math.abs(m[k][k]) < 1e-8) && (l < maxOrder))
            {
                SwitchColumns(k);
                l++;
            }
            if (l < maxOrder)
            {
                for (i = k; i <= maxOrder - 2; i++)
                {
                    if (!calcError)
                    {
                        if (Math.abs(m[i + 1][k]) > 1e-8)
                        {
                            for (l = k + 1; l <= maxOrder - 1; l++)
                            {
                                m[i + 1][l] = m[i + 1][l] * m[k][k] - m[k][l] * m[i + 1][k];
                            }
                            y[i + 1] = y[i + 1] * m[k][k] - y[k] * m[i + 1][k];
                            m[i + 1][k] = 0;
                        }
                    }
                }
            }
            else
                calcError = true;
        }
        return !calcError;
    }

    public void Solve(int empties)
    {
        int k, l;
        for (l = maxOrder - 1; l > maxOrder - empties - 1; l--)
            x[l] = 1.0;
        for (k = maxOrder - empties - 1; k >= 0; k--)
        {
            for (l = maxOrder - 1; l > k; l--)
            {
                y[k] = y[k] - x[l] * m[k][l];
            }
           x[k] = y[k] / m[k][k];
        }
        Reorder();
    }

}
